package org.jeecg.modules.adweb.server.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.modules.adweb.server.entity.AdwebServer;
import org.jeecg.modules.adweb.server.service.IAdwebServerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import java.util.Arrays;

/**
 * @Description: wp站点服务器配置信息
 * @Author: jeecg-boot
 * @Date: 2025-01-02
 * @Version: V1.0
 */
@Tag(name = "wp站点服务器配置信息")
@RestController
@RequestMapping("/adweb/adwebServer")
@Slf4j
public class AdwebServerController extends JeecgController<AdwebServer, IAdwebServerService> {
    @Autowired private IAdwebServerService adwebServerService;

    /**
     * 分页列表查询
     *
     * @param adwebServer
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    // @AutoLog(value = "wp站点服务器配置信息-分页列表查询")
    @Operation(summary = "wp站点服务器配置信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<AdwebServer>> queryPageList(
            AdwebServer adwebServer,
            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
            HttpServletRequest req) {
        QueryWrapper<AdwebServer> queryWrapper =
                QueryGenerator.initQueryWrapper(adwebServer, req.getParameterMap());
        Page<AdwebServer> page = new Page<AdwebServer>(pageNo, pageSize);
        IPage<AdwebServer> pageList = adwebServerService.page(page, queryWrapper);
        return Result.OK(pageList);
    }

    /**
     * 添加
     *
     * @param adwebServer
     * @return
     */
    @AutoLog(value = "wp站点服务器配置信息-添加")
    @Operation(summary = "wp站点服务器配置信息-添加")
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_server:add')")
    @PostMapping(value = "/add")
    public Result<String> add(@RequestBody AdwebServer adwebServer) {
        adwebServerService.save(adwebServer);
        return Result.OK("添加成功！");
    }

    /**
     * 编辑
     *
     * @param adwebServer
     * @return
     */
    @AutoLog(value = "wp站点服务器配置信息-编辑")
    @Operation(summary = "wp站点服务器配置信息-编辑")
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_server:edit')")
    @RequestMapping(
            value = "/edit",
            method = {RequestMethod.PUT, RequestMethod.POST})
    public Result<String> edit(@RequestBody AdwebServer adwebServer) {
        adwebServerService.updateById(adwebServer);
        return Result.OK("编辑成功!");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "wp站点服务器配置信息-通过id删除")
    @Operation(summary = "wp站点服务器配置信息-通过id删除")
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_server:delete')")
    @DeleteMapping(value = "/delete")
    public Result<String> delete(@RequestParam(name = "id", required = true) String id) {
        adwebServerService.removeById(id);
        return Result.OK("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "wp站点服务器配置信息-批量删除")
    @Operation(summary = "wp站点服务器配置信息-批量删除")
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_server:deleteBatch')")
    @DeleteMapping(value = "/deleteBatch")
    public Result<String> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        this.adwebServerService.removeByIds(Arrays.asList(ids.split(",")));
        return Result.OK("批量删除成功!");
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    // @AutoLog(value = "wp站点服务器配置信息-通过id查询")
    @Operation(summary = "wp站点服务器配置信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AdwebServer> queryById(@RequestParam(name = "id", required = true) String id) {
        AdwebServer adwebServer = adwebServerService.getById(id);
        if (adwebServer == null) {
            return Result.error("未找到对应数据");
        }
        return Result.OK(adwebServer);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param adwebServer
     */
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_server:exportXls')")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, AdwebServer adwebServer) {
        return super.exportXls(request, adwebServer, AdwebServer.class, "wp站点服务器配置信息");
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @PreAuthorize("@jps.requiresPermissions('adweb:adweb_server:importExcel')")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, AdwebServer.class);
    }
}
